@@ -221,7 +221,7 @@ int get_mmc_profile (int fd);
 int get_disc_size_cd (int fd);
 gint64 get_disc_size_dvd (int fd, int mmc_profile);
 int get_read_write_speed (int fd, int *read_speed, int *write_speed);
-int get_disc_status (int fd, int *empty, int *is_rewritable);
+int get_disc_status (int fd, int *empty, int *is_rewritable, int *is_blank);
 
 
 static void
@@ -286,8 +286,8 @@ nautilus_burn_drive_get_media_type_from_path_full (const char *device,
 	if (has_data) *has_data = FALSE;
 	if (has_audio) *has_audio = FALSE;
 
-	fd = open (device, O_RDWR|O_EXCL|O_NONBLOCK);
-	if (fd < 0) {
+	if ((fd = open (device, O_RDWR | O_EXCL | O_NONBLOCK)) < 0
+	    && (fd = open (device, O_RDONLY | O_EXCL | O_NONBLOCK)) < 0) {
 		if (errno == EBUSY) {
 			return NAUTILUS_BURN_MEDIA_TYPE_BUSY;
 		}
@@ -306,19 +306,16 @@ nautilus_burn_drive_get_media_type_from_path_full (const char *device,
 			close (fd);
 			return NAUTILUS_BURN_MEDIA_TYPE_ERROR;
 		} else {
-			int rewrite, empty;
-			if (get_disc_status (fd, &empty, &rewrite) == 0) {
+			int blank, rewrite, empty;
+			if (get_disc_status (fd, &empty, &rewrite, &blank) == 0) {
 				close (fd);
+
 				if (is_rewritable)
 					*is_rewritable = rewrite;
-				if (is_blank) {
-					/* FIXME */
-					*is_blank = TRUE;
-				}
-				if (has_data) {
-					/* FIXME */
-					*has_data = FALSE;
-				}
+				if (is_blank)
+					*is_blank = blank;
+				if (has_data)
+					*has_data = !blank;
 				if (has_audio) {
 					/* FIXME */
 					*has_audio = FALSE;
@@ -335,7 +332,16 @@ nautilus_burn_drive_get_media_type_from_path_full (const char *device,
 
 	close (fd);
 
-	switch (mmc_profile) {
+	if (is_blank)
+		*is_blank = mmc_profile & 0x10000;
+	if (has_data)
+		*has_data = !(mmc_profile & 0x10000);
+	if (has_audio) {
+		/* FIXME */
+		*has_audio = FALSE;
+	}
+
+	switch (mmc_profile & 0xFFFF) {
 	case -1:
 		g_assert_not_reached ();
 	case 0:		/* No Media */
@@ -568,8 +574,8 @@ nautilus_burn_drive_get_media_size_from_path (const char *device)
 
 	secs = 0;
 
-	fd = open (device, O_RDWR|O_EXCL|O_NONBLOCK);
-	if (fd < 0) {
+	if ((fd = open (device, O_RDWR | O_EXCL | O_NONBLOCK)) < 0
+	    && (fd = open (device, O_RDONLY | O_EXCL | O_NONBLOCK)) < 0) {
 		if (errno == EBUSY) {
 			return NAUTILUS_BURN_MEDIA_SIZE_BUSY;
 		}
@@ -579,7 +585,7 @@ nautilus_burn_drive_get_media_size_from_path (const char *device)
 	mmc_profile = get_mmc_profile (fd);
 
 	/* See nautilus_burn_drive_get_media_type_from_path for details */
-	switch (mmc_profile) {
+	switch (mmc_profile & 0xFFFF) {
 	case 0x9:
 	case 0xa:
 		secs = get_disc_size_cd (fd);
@@ -1055,7 +1061,10 @@ get_cd_scsi_id (const char *dev,
 	} m_idlun;
 	
 	devfile = g_strdup_printf ("/dev/%s", dev);
-	fd = open(devfile, O_RDWR | O_NONBLOCK);
+	fd = open (devfile, O_RDWR | O_NONBLOCK);
+	if (fd < 0)
+		fd = open (devfile, O_RDONLY | O_NONBLOCK);
+
 	g_free (devfile);
 
 	/* Avoid problems with Valgrind */
@@ -1114,8 +1123,8 @@ get_device_max_read_speed (char *device)
 
 	max_speed = -1;
 
-	fd = open (device, O_RDWR|O_EXCL|O_NONBLOCK);
-	if (fd < 0) {
+	if ((fd = open (device, O_RDWR | O_EXCL | O_NONBLOCK)) < 0
+	    && (fd = open (device, O_RDONLY | O_EXCL | O_NONBLOCK)) < 0) {
 		return -1;
 	}
 
@@ -1136,8 +1145,8 @@ get_device_max_write_speed (char *device)
 
 	max_speed = -1;
 
-	fd = open (device, O_RDWR|O_EXCL|O_NONBLOCK);
-	if (fd < 0) {
+	if ((fd = open (device, O_RDWR | O_EXCL | O_NONBLOCK)) < 0
+	    && (fd = open (device, O_RDONLY | O_EXCL | O_NONBLOCK)) < 0) {
 		return -1;
 	}
 
